<?php
/**
* MySQL_Connection V2
* คลาสที่จะช่วยให้คุณเขียนโปรแกรมเชื่อมต่อกับฐาน MySQL ได้สะดวกและปลอดภัยขึ้น
* Copyright (c) 2014, phpinfo.in.th (http://www.phpinfo.in.th)
*/
class MySQL_Connection
{
	protected static $options = array(
		'mysqli.default_host',
		'mysqli.default_user',
		'mysqli.default_pw',
		null,
		'mysqli.default_port',
		'mysqli.default_socket',
	);

	private $_debug = true;
	private $_tablePrefix = '';
	private $_defaultCharset = 'utf8';
	private $_affectedRows;
	private $_numRows;
	private $args;
	private $c;
	private $r;
	private $s = array();
	private $d;
	private $i;

	public function __construct(
		$host = null,
		$username = null,
		$password = null,
		$db = null,
		$port = null,
		$socket = null
	) {
		$args = array($host, $username, $password, $db, $port, $socket);
		foreach (self::$options as $i => $default) {
			if (!isset($args[$i]) && isset($default)) {
				$args[$i] = ini_get($default);
			}
		}
		$this->args = $args;
	}

	protected function replaceStringCallback($matches)
	{
		if (isset($matches[3])) {
			$nullable = isset($matches[3][1]);
			if (isset($matches[4])) {
				$value = isset($this->d[$matches[4]])
					? $this->d[$matches[4]]
					: null;
			} else {
				$value = isset($this->d[$this->i])
					? $this->d[$this->i++]
					: null;
			}
			if (is_array($value)) {
				switch ($matches[3][0]) {
					case '%':
						return $this->replaceString(implode(',', $value));
					case 'i':
						foreach ($value as &$ref) {
							$ref = implode(
								'`.`',
								explode('.', preg_replace('/`/u', '``', $ref), 3)
							);
						}
						return '`' . implode('`,`', $value) . '`';
					case 'b':
						foreach ($value as &$ref) {
							$ref = $nullable && $ref === null
								? 'NULL'
								: ((bool) $ref ? 'TRUE' : 'FALSE');
						}
						return implode(',', $value);
					case 'd':
						foreach ($value as &$ref) {
							$ref = $nullable && $ref === null
								? 'NULL'
								: (int)$ref;
						}
						return implode(', ', $value);
					case 'f':
						foreach ($value as &$ref) {
							$ref = $nullable && $ref === null
								? 'NULL'
								: (float)$ref;
						}
						return implode(', ', $value);
					case 's':
						foreach ($value as &$ref) {
							$ref = $nullable && $ref === null
								? 'NULL'
								: $this->c->real_escape_string($ref);
						}
						return "'" . implode("','", $value) . "'";
				}
			} else {
				switch ($matches[3][0]) {
					case '%':
						return $this->replaceString($value);
					case 'i':
						return '`'
							. implode(
								'`.`',
								explode('.', preg_replace('/`/u', '``', $value), 3)
							)
							. '`';
					case 'b':
						return $nullable && $value === null
							? 'NULL'
							: ((bool) $value ? 'TRUE' : 'FALSE');
					case 'd':
						return $nullable && $value === null
							? 'NULL'
							: (int)$value;
					case 'f':
						return $nullable && $value === null
							? 'NULL'
							: (float)$value;
					case 's':
						return $nullable && $value === null
							? 'NULL'
							: "'{$this->c->real_escape_string($value)}'";
				}
			}
		} elseif (isset($matches[2])) {
			return "`{$this->_tablePrefix}{$matches[2]}`";
		} elseif (isset($matches[1])) {
			return "`{$matches[1]}`";
		}
		return $matches[0];
	}

	public function connect()
	{
		if (isset($this->c)) {
			return;
		}
		$this->c = mysqli_init();
		$args = func_get_args();
		@call_user_func_array(array($this->c, 'connect'), $args + $this->args);
		if ($this->c->connect_error) {
			$exception = new MySQL_Exception(
				$this->c->connect_error,
				$this->c->connect_errno
			);
			$this->c = null;
			if ($this->_debug) {
				$exception->debug();
			}
			throw $exception;
		}
		$this->c->set_charset($this->_defaultCharset);
	}

	public function selectDb($db)
	{
		$this->connect();
		$this->c->select_db($db);
		if ($this->c->error) {
			$exception = new MySQL_Exception($this->c->error, $this->c->errno);
			if ($this->_debug) {
				$exception->debug();
			}
			throw $exception;
		}
	}

	public function ping()
	{
		$this->connect();
		return $this->c->ping();
	}

	public function close()
	{
		if (!isset($this->c)) {
			return;
		}
		$this->free();
		$return = $this->c->close();
		$this->c = null;
		return $return;
	}

	public function escapeString($value)
	{
		if (!isset($this->c)) {
			$this->connect();
		}
		return $this->c->real_escape_string($value);
	}

	public function replaceString()
	{
		if (!isset($this->c)) {
			$this->connect();
		}
		$args = func_get_args();
		$query = array_shift($args);
		if ($args) {
			if (!isset($args[1])) {
				$args = is_array($args[0]) ? $args[0] : array($args[0]);
			}
			if (isset($this->d)) {
				$this->s[] = array($this->d, $this->i);
			}
			$this->d = $args;
			$this->i = 0;
		}
		$result = preg_replace_callback(
			'/\'(?>\\\\\'|\'\'|[^\']+)*\'|"(?>\\\\"|""|[^"]+)*"|`(?>``|[^`]+)*`|\[([^\s\]]+)\]|\<([^\s\>]+)\>|%(%|i|(?>s|b|d|f)n?)?(?>\[([^\]]+)\])?|%/u',
			array($this, 'replaceStringCallback'),
			$query
		);
		if ($args) {
			if (!empty($this->s)) {
				list($this->d, $this->i) = array_pop($this->s);
			} else {
				$this->d = null;
			}
		}
		return $result;
	}

	public function query($query, $params = null)
	{
		$this->connect();
		$this->free();
		$result = $this->c->query($actualQuery = $this->replaceString($query, $params));
		if ($result === false) {
			$exception = new MySQL_Exception(
				$this->c->error,
				$this->c->errno,
				$query,
				$actualQuery
			);
			if ($this->_debug) {
				$exception->debug();
			}
		}
		if ($result instanceof mysqli_result) {
			$this->r = $result;
			return $this->_numRows = $this->_affectedRows = $this->c->affected_rows;
		}
		$this->_numRows = 0;
		return $this->_affectedRows = $this->c->affected_rows;
	}

	public function queryAndFetch($query, $params = null, $columnKey = null)
	{
		$this->query($query, $params);
		$row = $this->fetch($columnKey);
		$this->free();
		return $row;
	}

	public function queryAndFetchAll($query, $params = null, $columnKey = null, $indexKey = null)
	{
		$this->query($query, $params);
		return $this->fetchAll($columnKey, $indexKey);
	}

	public function queryValue($query, $params = null)
	{
		if ($this->query($query, $params)) {
			$row = $this->fetch();
			$this->free();
			return $row[key($row)];
		}
	}

	public function fetch($columnKey = null)
	{
		if (!isset($this->r)) {
			return;
		}
		if (!($row = $this->r->fetch_assoc())) {
			$this->free();
		} else {
			return isset($columnKey)
				? (isset($row[$columnKey])
					? $row[$columnKey]
					: null)
				: $row;
		}
	}

	public function fetchAll($columnKey = null, $indexKey = null)
	{
		if (!isset($this->r)) {
			return;
		}
		$rows = array();
		if (($row = $this->r->fetch_assoc())) {
			if (isset($columnKey)) {
				if (array_key_exists($columnKey, $row)) {
					if (isset($indexKey) && array_key_exists($indexKey, $row)) {
						do {
							$rows[$row[$indexKey]] = $row[$columnKey];
						} while (($row = $this->r->fetch_assoc()));
					} else {
						do {
							$rows[] = $row[$columnKey];
						} while (($row = $this->r->fetch_assoc()));
					}
				}
			} elseif (isset($indexKey) && array_key_exists($indexKey, $row)) {
				do {
					$rows[$row[$indexKey]] = $row;
				} while (($row = $this->r->fetch_assoc()));
			} else {
				do {
					$rows[] = $row;
				} while (($row = $this->r->fetch_assoc()));
			}
		}
		$this->free();
		return $rows;
	}

	public function free()
	{
		if (!isset($this->r)) {
			return;
		}
		$this->r->free();
		$this->r = null;
	}

	public function __get($name)
	{
		if (isset($this->{$prop = '_' . $name})) {
			return $this->$prop;
		}
		if ($name === 'charset') {
			$this->connect();
			return $this->c->character_set_name();
		}
		static $map = array(
			'error' => 'error',
			'errno' => 'errno',
			'insertId' => 'insert_id',
		);
		if (isset($map[$name])) {
			return $this->c->{$map[$name]};
		}
	}

	public function __set($name, $value)
	{
		if (isset($this->{$prop = '_' . $name})) {
			$this->$prop = $name === 'debug'
				? (bool)$value
				: (string)$value;
		}
		if ($name === 'charset') {
			$this->connect();
			$this->c->set_charset($value);
			if ($this->c->error) {
				$exception = new MySQL_Exception($this->c->error, $this->c->errno);
				if ($this->_debug) {
					$exception->debug();
				}
				throw $exception;
			}
		}
	}
}

class MySQL_Exception extends Exception
{
	public static function formatMessage($message, $width = 100)
	{
		$width = max(40, (int)$width);
		if (!preg_match_all('/^\t+/mu', $message, $matches)) {
			$matches[0][0] = '';
		}
		sort($matches[0]);
		$message = trim(
			preg_replace(
				'/\t/u',
				'    ',
				preg_replace("/^{$matches[0][0]}/mu", '', $message)
			)
		);
		if (preg_match_all("/\\n?(.{0,{$width}})/u", $message, $matches, PREG_SET_ORDER)) {
			foreach ($matches as $match) {
				$lines[] = $match[1];
			}
			return implode("\n", $lines);
		}
		return $message;
	}

	private $sourceQuery;
	private $actualQuery;

	public function __construct($message, $code, $sourceQuery = null, $actualQuery = null)
	{
		parent::__construct((string)$message, (int)$code);
		$this->sourceQuery = $sourceQuery;
		$this->actualQuery = $actualQuery;
	}

	public function getDebugMessage($width = 100)
	{
		$hr = str_repeat('-', $width = max(40, (int)$width));
		return self::formatMessage(
			isset($this->sourceQuery)
				? sprintf(
					"{$hr}\nMySQL error:\n{$hr}\n\n#%s - %s\n\n{$hr}\nSource query:\n{$hr}\n\n%s\n\n{$hr}\nActual query:\n{$hr}\n\n%s\n\n{$hr}\nTrace:\n{$hr}\n\n%s",
					$this->getCode(),
					$this->getMessage(),
					$this->sourceQuery,
					$this->actualQuery,
					$this->getTraceAsString()
				)
				: sprintf(
					"{$hr}\nMySQL error:\n{$hr}\n\n#%s - %s\n\n{$hr}\nTrace:\n{$hr}\n\n%s",
					$this->getCode(),
					$this->getMessage(),
					$this->getTraceAsString()
				),
			$width
		);
	}

	public function debug($width = 100)
	{
		if (!headers_sent()) {
			header('Content-Type: text/plain; charset=utf-8', true);
			while (ob_get_level()) {
				ob_end_clean();
			}
		}
		exit($this->getDebugMessage($width));
	}
}
